<?php

namespace Moscow\RUSComing\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moscow\RUSComing\Http\Requests\IndexRequest;
use Moscow\RUSComing\Interfaces\IndexRepositoryInterface;
use Moscow\RUSComing\Models\Index;

/**
 * Resource controller class for index.
 */
class IndexResourceController extends BaseController
{

    /**
     * Initialize index resource controller.
     *
     * @param type IndexRepositoryInterface $index
     *
     * @return null
     */
    public function __construct(IndexRepositoryInterface $index)
    {
        parent::__construct();
        $this->repository = $index;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moscow\RUSComing\Repositories\Criteria\IndexResourceCriteria::class);
    }

    /**
     * Display a list of index.
     *
     * @return Response
     */
    public function index(IndexRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moscow\RUSComing\Repositories\Presenter\IndexPresenter::class)
                ->$function();
        }

        $indices = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('r_u_s-coming::index.names'))
            ->view('r_u_s-coming::index.index', true)
            ->data(compact('indices', 'view'))
            ->output();
    }

    /**
     * Display index.
     *
     * @param Request $request
     * @param Model   $index
     *
     * @return Response
     */
    public function show(IndexRequest $request, Index $index)
    {

        if ($index->exists) {
            $view = 'r_u_s-coming::index.show';
        } else {
            $view = 'r_u_s-coming::index.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('r_u_s-coming::index.name'))
            ->data(compact('index'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new index.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(IndexRequest $request)
    {

        $index = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('r_u_s-coming::index.name')) 
            ->view('r_u_s-coming::index.create', true) 
            ->data(compact('index'))
            ->output();
    }

    /**
     * Create new index.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(IndexRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $index                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('r_u_s-coming::index.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('r_u_s-coming/index/' . $index->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/r_u_s-coming/index'))
                ->redirect();
        }

    }

    /**
     * Show index for editing.
     *
     * @param Request $request
     * @param Model   $index
     *
     * @return Response
     */
    public function edit(IndexRequest $request, Index $index)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('r_u_s-coming::index.name'))
            ->view('r_u_s-coming::index.edit', true)
            ->data(compact('index'))
            ->output();
    }

    /**
     * Update the index.
     *
     * @param Request $request
     * @param Model   $index
     *
     * @return Response
     */
    public function update(IndexRequest $request, Index $index)
    {
        try {
            $attributes = $request->all();

            $index->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('r_u_s-coming::index.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('r_u_s-coming/index/' . $index->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('r_u_s-coming/index/' . $index->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the index.
     *
     * @param Model   $index
     *
     * @return Response
     */
    public function destroy(IndexRequest $request, Index $index)
    {
        try {

            $index->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('r_u_s-coming::index.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('r_u_s-coming/index/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('r_u_s-coming/index/' . $index->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple index.
     *
     * @param Model   $index
     *
     * @return Response
     */
    public function delete(IndexRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('r_u_s-coming::index.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('r_u_s-coming/index'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/r_u_s-coming/index'))
                ->redirect();
        }

    }

    /**
     * Restore deleted indices.
     *
     * @param Model   $index
     *
     * @return Response
     */
    public function restore(IndexRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('r_u_s-coming::index.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/r_u_s-coming/index'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/r_u_s-coming/index/'))
                ->redirect();
        }

    }

}
